;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'mouseme '("ess-mous" "mouseme"))
(custom-add-loads 'shell '("msdos"))
(custom-add-loads 'ess-S '("ess-cust"))
(custom-add-loads 'ess-sas '("essa-sas" "essl-sas"))
(custom-add-loads 'ess-edit '("ess-cust"))
(custom-add-loads 'hypermedia '("mouseme"))
(custom-add-loads 'ess-proc '("ess-cust"))
(custom-add-loads 'ess '("ess-cust" "ess-menu"))
(custom-add-loads 'local '("ess-cust"))
(custom-add-loads 'ess-hooks '("ess-cust"))
(custom-add-loads 'ess-XLS '("ess-cust"))
(custom-add-loads 'ess-SAS '("ess-cust"))
(custom-add-loads 'ess-Stata '("ess-cust"))
(custom-add-loads 'ess-OMG '("ess-cust"))
(custom-add-loads 'ess-command '("ess-cust"))
(custom-add-loads 'ess-bugs '("essl-bug"))

;;; custom-load.el ends here
