(defvar semantic-toplevel-sas-bovine-table
  nil
  "Table for use with semantic for parsing R.")

(defvar semantic-sas-keyword-table
  nil
  "Keyword table for use with semantic for parsing R.")

(defvar semantic-default-sas-setup 
  nil
  " test .")




